package com.faraphel.tasks_valider.connectivity.task

import com.faraphel.tasks_valider.connectivity.task.api.TaskSessionManagerApi
import com.faraphel.tasks_valider.connectivity.task.session.TaskSessionManager
import com.faraphel.tasks_valider.database.TaskDatabase
import com.faraphel.tasks_valider.database.api.TaskDatabaseApi
import fi.iki.elonen.NanoHTTPD


/**
 * A server to handle the task API to allow clients to interact with the database.
 * @param port the port of the server
 * @param database the database to interact with
 */
class TaskServer(
    private val port: Int,
    private val database: TaskDatabase
) : NanoHTTPD(port) {
    private val sessionManager = TaskSessionManager()  ///< the session manager

    private val sessionManagerApi = TaskSessionManagerApi(this.sessionManager)  ///< the api of the session manager
    private val databaseApi = TaskDatabaseApi(this.database)  ///< the api of the database

    /**
     * Handle an API request
     * @param httpSession the http session
     */
    override fun serve(httpSession: IHTTPSession): Response {
        // get the session data of the client
        val taskSession = this.sessionManager.getOrCreateSessionData(httpSession)

        // parse the url
        val uri: String = httpSession.uri.substring(1)  // remove the first slash
        val path = uri.split("/").toMutableList()

        // get the type of the request from the uri
        val requestType = path.removeFirstOrNull()
            ?: return newFixedLengthResponse(
                NanoHTTPD.Response.Status.BAD_REQUEST,
                "text/plain",
                "Missing request type"
            )

        // get the response from the correct part of the application
        val response = when (requestType) {
            // session requests
            "sessions" -> this.sessionManagerApi.handleRequest(taskSession, httpSession, path)
            // entities requests
            "entities" -> return this.databaseApi.handleRequest(taskSession, httpSession, path)
            // invalid requests
            else ->
                newFixedLengthResponse(
                    NanoHTTPD.Response.Status.BAD_REQUEST,
                    "text/plain",
                    "Unknown request type"
                )
        }

        // wrap additional information in the response
        return this.sessionManager.responseSetSessionData(response, httpSession.cookies)
    }

    /**
     * Start the server with the default configuration
     */
    override fun start() = super.start(SOCKET_READ_TIMEOUT, false)
}
